PROGRAM main
    USE HDF5
    USE ISO_FORTRAN_ENV
    
    ! Local variables
    INTEGER(HID_T)  :: fileh, dcpl, grouph, attrh, filespace
    INTEGER(HID_T) :: vspace, src_space, dset
    INTEGER(HSIZE_T) :: vdims(3), dims(3), adims(1), maxdims(3), offset(3)
    INTEGER(kind=int32) :: ierr
    INTEGER :: i
    CHARACTER(len=32) :: dsetname
    INTEGER, ALLOCATABLE :: dsetLength(:)
    LOGICAL :: cannotMakeVDS
    
    INTEGER(kind=int32), ALLOCATABLE :: finished(:)
    
    INTEGER :: nRuns = 2
    INTEGER :: nVars = 4
    INTEGER :: nsolint = 1
    CHARACTER(len=32) :: filename = "solintData.h5"
    CHARACTER(len=32) :: groupname
    
    CALL h5open_f(ierr)
    
    ! Initial dimensions and offset of VDS dataset
    vdims(1) = nVars
    vdims(2) = nsolint
    vdims(3) = 0
    offset = 0
    
    ALLOCATE(dsetLength(nRuns))
    ALLOCATE(finished(nRuns))
    finished = 0
    cannotMakeVDS = .FALSE.
    
    ! Open source datasets and check dimensions
    CALL h5fopen_f(filename, H5F_ACC_RDONLY_F, fileh, ierr)
    DO i = 1,nRuns
        WRITE(groupname, '(A,I0)') "run-", i
        
        CALL h5gopen_f(fileh, groupname, grouph, ierr)
        
        ! Check if group is finished
        adims(1) = 1
        CALL h5aopen_f(grouph, 'finished', attrh, ierr)
        CALL h5aread_f(attrh, H5T_NATIVE_INTEGER, finished(i), dims, ierr)
        CALL h5aclose_f(attrh, ierr)
        IF (finished(i) /= 1) THEN
            CALL h5gclose_f(grouph, ierr)
            CYCLE
        END IF
        
        CALL h5dopen_f(grouph, "values", dset, ierr)
        CALL h5dget_space_f(dset, filespace, ierr)
        CALL H5Sget_simple_extent_dims_f(filespace, dims, maxdims, ierr)
        
        IF (dims(1) /= vdims(1) .OR. dims(2) /= vdims(2)) THEN
            cannotMakeVDS = .TRUE.
            EXIT
        END IF
        
        dsetLength(i) = dims(3)
        vdims(3) = vdims(3) + dims(3)
        
        CALL h5sclose_f(filespace, ierr)
        CALL h5dclose_f(dset, ierr)
        CALL h5gclose_f(grouph, ierr)
    END DO
    CALL h5fclose_f(fileh, ierr)
    
    IF (cannotMakeVDS) THEN
        write(*,*) "H5SOLINT ERRR: Cannot make VDS."
        write(*,*) ""
        RETURN
    END IF
    
    ! Create VDS file, truncate if it already exists
    CALL h5fcreate_f("solint.h5", H5F_ACC_TRUNC_F, fileh, ierr)
    
    ! VDS dataset creation property list
    CALL h5pcreate_f(H5P_DATASET_CREATE_F, dcpl, ierr);
    
    ! Create VDS dataspace and set mapping for values array
    CALL h5screate_simple_f(3, vdims, vspace, ierr)
    
    offset = 0
    dims(1) = nVars
    dims(2) = nsolint
    DO i = 1,nRuns
        IF (finished(i) == 1) THEN
            WRITE(dsetname, '(A,I0,A)') "/run-", i, "/values"
            dims(3) = dsetLength(i)
            
            CALL h5screate_simple_f(3, dims, src_space, ierr)
            CALL h5sselect_hyperslab_f(vspace, H5S_SELECT_SET_F, offset, dims, ierr)
            CALL h5pset_virtual_f(dcpl, vspace, filename, dsetname, src_space, ierr)
            CALL h5sclose_f(src_space, ierr)
            
            offset(3) = offset(3) + dims(3)
        END IF
    END DO
    
    CALL h5pset_fill_value_f(dcpl, H5T_NATIVE_REAL, great, ierr);
    CALL h5dcreate_f(fileh, "values", H5T_NATIVE_REAL, vspace, dset, ierr, dcpl_id=dcpl)
    CALL h5dclose_f(dset, ierr)
    CALL h5sclose_f(vspace, ierr)
    
    
    ! Create VDS dataspace and set mapping for time array
    CALL h5screate_simple_f(1, vdims(3), vspace, ierr)
    
    offset(1) = 0
    DO i = 1,nRuns
        IF (finished(i) == 1) THEN
            WRITE(dsetname, '(A,I0,A)') "/run-", i, "/time"
            dims(1) = dsetLength(i)
            
            CALL h5screate_simple_f(1, dims, src_space, ierr)
            CALL h5sselect_hyperslab_f(vspace, H5S_SELECT_SET_F, offset, dims, ierr)
            CALL h5pset_virtual_f(dcpl, vspace, filename, dsetname, src_space, ierr)
            CALL h5sclose_f(src_space, ierr)
            
            offset(1) = offset(1) + dims(1)
        END IF
    END DO
    
    CALL h5pset_fill_value_f(dcpl, H5T_NATIVE_REAL, -1.0, ierr);
    CALL h5dcreate_f(fileh, "time", H5T_NATIVE_REAL, vspace, dset, ierr, dcpl_id=dcpl)
    CALL h5dclose_f(dset, ierr)
    CALL h5sclose_f(vspace, ierr)
    
    
    ! Create VDS dataspace and set mapping for itstep array
    CALL h5screate_simple_f(1, vdims(3), vspace, ierr)
    
    offset(1) = 0
    DO i = 1,nRuns
        IF (finished(i) == 1) THEN
            WRITE(dsetname, '(A,I0,A)') "/run-", i, "/itstep"
            dims(1) = dsetLength(i)
            
            CALL h5screate_simple_f(1, dims, src_space, ierr)
            CALL h5sselect_hyperslab_f(vspace, H5S_SELECT_SET_F, offset, dims, ierr)
            CALL h5pset_virtual_f(dcpl, vspace, filename, dsetname, src_space, ierr)
            CALL h5sclose_f(src_space, ierr)
            
            offset(1) = offset(1) + dims(1)
        END IF
    END DO
    
    CALL h5pset_fill_value_f(dcpl, H5T_NATIVE_INTEGER, -1, ierr);
    CALL h5dcreate_f(fileh, "itstep", H5T_NATIVE_INTEGER, vspace, dset, ierr, dcpl_id=dcpl)
    CALL h5dclose_f(dset, ierr)
    CALL h5sclose_f(vspace, ierr)
    
    
    ! Create link to coordinates dataset
    CALL h5lcreate_external_f(filename, "/run-1/coordinates", fileh, "coordinates", ierr)
    
    CALL h5pclose_f(dcpl, ierr)
    CALL h5fclose_f(fileh, ierr)
    
    DEALLOCATE(finished)
    DEALLOCATE(dsetLength)
    
    CALL h5close_f(ierr)
    
END PROGRAM
